"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFnContext = void 0;
/**
 * 获取单个数据源云函数的context
 */
const tcb_1 = require("../tcb");
const utils_1 = require("../utils");
const json_transform_1 = require("../http-method/json-transform");
const httpAuthProvider = __importStar(require("../http-method/auth"));
const call_data_source_1 = require("./call-data-source");
// 数据源类型, 0 空 1 腾讯文档 2 腾讯会议 3 企业微信 4 微信电商
const HTTP_SOURCE_TYPES = [
    '',
    'tencent-docs',
    'tencent-meeting',
    'weixin-work',
    'weixin-shop',
];
function getFnContext(config) {
    var _a;
    // const envObj = getCloudbaseContext(config.rawContext, config.isTestMode);
    const parsedContext = tcb_1.tcb.parseContext(config.rawContext);
    const envTypeInfo = getEnvType(config);
    // 目前仅内建数据源才有数据库
    const hasDb = ((_a = config.dsConfig) === null || _a === void 0 ? void 0 : _a.type) === 'database';
    const envInfo = getEnvInfo(config);
    const vars = getVarsInfo(config);
    // 自定义操作的context
    const cfContext = {
        cloudbase: tcb_1.tcb,
        app: tcb_1.app,
        auth: tcb_1.auth,
        callDataSource: call_data_source_1.createCallDataSource({
            handlers: config.handlers,
            isPreview: envTypeInfo.isPreview,
            isTestMode: config.isTestMode,
            dsConfig: config.dsConfig,
            dsNameIdMap: config.dsNameIdMap,
        }),
        vars,
        httpAuth: getAuthProvider(config.dsConfig),
        env: envInfo,
        envInfo,
        database: tcb_1.db,
        dataSourceProfile: config.dsConfig,
        weixin: getWxCloudInfo(config.params),
        defaultParams: config.params.defaultParams,
        collection: hasDb && envInfo.dataSourceFullName
            ? tcb_1.db.collection(envInfo.dataSourceFullName)
            : undefined,
        wedaTarget: config.params.wedaTarget,
    };
    utils_1.setConfig({
        wedaTarget: cfContext.wedaTarget,
        requestId: parsedContext.requestId,
        envType: envTypeInfo.envType,
        proxy: config.params.proxy,
        userSource: config.params.userSource,
    });
    return cfContext;
}
exports.getFnContext = getFnContext;
function getAuthProvider(dsConfig) {
    var _a, _b;
    if (!dsConfig || dsConfig.type !== 'cloud-integration')
        return;
    // @ts-ignore
    const providerName = (dsConfig.source && HTTP_SOURCE_TYPES[dsConfig.source])
        || ((_a = dsConfig.configuration) === null || _a === void 0 ? void 0 : _a.provider)
        || ((_b = dsConfig.schema) === null || _b === void 0 ? void 0 : _b.provider);
    if (!providerName)
        return;
    console.log('provider', providerName);
    // @ts-ignore
    const authProvider = providerName ? httpAuthProvider[utils_1.dash2smallCamelCase(providerName)] : undefined;
    if (!authProvider)
        return;
    return Object.assign(Object.assign({}, authProvider), { getAuthData: authProvider.getAuthData.bind(null, dsConfig.id) });
}
/**
 * 获取环境变量信息
 */
function getEnvInfo(config) {
    var _a;
    const envTypeInfo = getEnvType(config);
    const envObj = utils_1.getCloudbaseContext(config.rawContext, config.isTestMode);
    const dsFullName = config.dsConfig && utils_1.getDsFullName(config.dsConfig, envTypeInfo.isPreview);
    return Object.assign(Object.assign({}, envTypeInfo), { wedaAppId: config.params.wedaAppId, openId: envObj.WX_OPENID, 
        // @ts-ignore
        fromOpenId: envObj.WX_FROM_OPENID, appId: envObj.WX_APPID, 
        // @ts-ignore
        fromAppId: envObj.WX_FROM_APPID, uid: envObj.TCB_UUID, customUserId: envObj.TCB_CUSTOM_USER_ID || '', isAnonymous: envObj.TCB_ISANONYMOUS_USER === 'true', dataSourceName: (_a = config.dsConfig) === null || _a === void 0 ? void 0 : _a.name, dataSourceFullName: dsFullName, envId: envObj.TCB_ENV || envObj.SCF_NAMESPACE });
}
/**
 * 获取公共变量信息
 */
function getVarsInfo(config) {
    var _a, _b, _c, _d;
    if (!((_b = (_a = config.dsConfig) === null || _a === void 0 ? void 0 : _a.configuration) === null || _b === void 0 ? void 0 : _b.vars))
        return {};
    return (json_transform_1.transformJSONWithTemplate({}, (_d = (_c = config.dsConfig) === null || _c === void 0 ? void 0 : _c.configuration) === null || _d === void 0 ? void 0 : _d.vars) || {});
}
/**
 * 获取环境类型信息, 返回 { isPreview, envType }
 *  优先使用 config.isPreview 判断环境类型, 再使用环境变量, 最后才使用 params 中的 envType
 */
function getEnvType(config) {
    let isPreview;
    let { envType } = config.params;
    if (typeof config.isPreview === 'boolean') {
        isPreview = config.isPreview;
    }
    if (isPreview === undefined && process.env.IS_PREVIEW) {
        isPreview = true;
    }
    if (isPreview === undefined) {
        if (envType) {
            isPreview = envType === 'pre';
            // console.log('isPreview', isPreview, envType)
            return {
                isPreview,
                envType,
            };
        }
        isPreview = true;
    }
    envType = isPreview ? 'pre' : 'prod';
    return {
        isPreview,
        envType,
    };
}
function getWxCloudInfo(params) {
    const wxReg = /^wx_/;
    const wxKeys = Object.keys(params).filter(k => wxReg.test(k));
    if (!wxKeys.length)
        return null;
    const weixin = {};
    return wxKeys.reduce((acc, cur) => {
        acc[cur.replace(wxReg, '')] = params[cur];
        return acc;
    }, weixin);
}
